"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.all = void 0;
const tslib_1 = require("tslib");
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const compare_versions_1 = require("compare-versions");
const createRule = experimental_utils_1.ESLintUtils.RuleCreator((ruleName) => ruleName);
// ts 3.9 introduce ts-expect-error comment.
// @see https://devblogs.microsoft.com/typescript/announcing-typescript-3-9-beta/
const defaultType = compare_versions_1.compare(typescript_1.default.versionMajorMinor, "3.9", ">=") ? "ts-expect-error" : "ts-ignore";
const defaultOptions = {
    tsCommentType: defaultType,
    reportOnly: false,
};
exports.all = createRule({
    name: "all",
    meta: {
        type: "suggestion",
        docs: {
            description: "show tsc error plugin.",
            category: "Best Practices",
            recommended: false,
            requiresTypeChecking: true,
        },
        messages: {
            // see: https://eslint.org/docs/developer-guide/working-with-rules#using-message-placeholders
            message: "{{ message }}",
        },
        schema: [
            {
                type: "object",
                properties: {
                    tsCommentType: {
                        type: "string",
                        enum: ["ts-ignore", "ts-expect-error"],
                    },
                    reportOnly: {
                        type: "boolean",
                    },
                },
                additionalProperties: false,
            },
        ],
        fixable: "code",
    },
    defaultOptions: [defaultOptions],
    create(context, [options]) {
        const { program } = experimental_utils_1.ESLintUtils.getParserServices(context);
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        const emitResults = program.emit(undefined, () => { });
        const allDiagnostics = [
            ...typescript_1.default.getPreEmitDiagnostics(program, program.getSourceFile(context.getFilename())),
            ...emitResults.diagnostics,
        ];
        return {
            "Program:exit": function () {
                allDiagnostics.forEach((diagnostic) => {
                    if (diagnostic.file) {
                        const { line: startLine, character: startCharacter } = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
                        const { line: endLine, character: endCharacter } = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start + diagnostic.length);
                        const message = typescript_1.default.flattenDiagnosticMessageText(diagnostic.messageText, "\n");
                        const firstOfLineToken = diagnostic.file.getPositionOfLineAndCharacter(startLine, 0);
                        const { reportOnly } = options;
                        // eslint line loc and ts line loc has difference.
                        // should we use tsNodeToESTreeNodeMap?
                        const esStartLine = startLine + 1;
                        const esEndLine = endLine + 1;
                        // TODO: filter error via typescript error codes.
                        context.report({
                            loc: {
                                start: {
                                    line: esStartLine,
                                    column: startCharacter,
                                },
                                end: {
                                    line: esEndLine,
                                    column: endCharacter,
                                },
                            },
                            messageId: "message",
                            data: {
                                message,
                            },
                            fix: (() => {
                                if (reportOnly) {
                                    return null;
                                }
                                return (fixer) => {
                                    const { tsCommentType = "ts-ignore" } = options;
                                    const comment = `/** ${message} */
// @${tsCommentType} with above original error.
`;
                                    return fixer.insertTextBeforeRange([firstOfLineToken, firstOfLineToken], comment);
                                };
                            })(),
                        });
                    }
                });
            },
        };
    },
});
//# sourceMappingURL=all.js.map